import java.awt.Color;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.FileNotFoundException;
import java.io.IOException;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JColorChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/*
 This class holds all of the shape buttons used for JavaPaint
 holds the three shape buttons in the left section of the Frame
 also has a reference back to the main program to set the current brush

 Danny Nguyen
 */

public class ButtonPanel extends JPanel implements ActionListener,ChangeListener, ItemListener {
	//initializing all of the data members
	private JButton squareButton = new JButton("Square");
	private JButton circleButton = new JButton("Circle");
	private JButton lineButton = new JButton("Line");
	private JButton sortButton = new JButton("Sort");
	private JButton undoButton = new JButton("undo");
	private JButton redoButton = new JButton("redo");
	private JSlider strokeSlider = new JSlider(1, 100, 1);//slider limited to 1-100 and start value is 1
	private JCheckBox fillCheck = new JCheckBox("fill");
	private JButton colorButton = new JButton("Color Picker");
	private JButton saveButton = new JButton("save");
	private JButton loadButton = new JButton("load");


	private JavaPaint owner;// reference to JavaPaint.java
	private DrawPanel canvas;// reference to DrawPanel.java

	// Main Constructor
	// takes a reference to the main program to get brush
	// also takes a reference to the draw panel to sort the shapes
	public ButtonPanel(JavaPaint ref, DrawPanel ref2) {
		owner = ref;// reference to the main program
		canvas = ref2;// reference to the draw panel
		setLayout(new GridLayout(7, 2));// arranges the buttons into 2 columns
		// places the buttons into the column
		add(saveButton);
		add(loadButton);
		add(undoButton);
		add(redoButton);
		add(squareButton);
		add(circleButton);
		add(lineButton);
		add(sortButton);
		add(strokeSlider);
		add(fillCheck);
		add(colorButton);
		
		// adds actionlistener all buttons
		saveButton.addActionListener(this);
		loadButton.addActionListener(this);
		redoButton.addActionListener(this);
		colorButton.addActionListener(this);
		fillCheck.addItemListener(this);
		strokeSlider.addChangeListener(this);
		undoButton.addActionListener(this);
		sortButton.addActionListener(this);
		squareButton.addActionListener(this);
		circleButton.addActionListener(this);
		lineButton.addActionListener(this);
	}

	// Listens for button presses
	// sets the current brush in JavaPaint to the shape
	// Corresponding to the button pressed
	@Override
	public void actionPerformed(ActionEvent e) {
		JButton source = (JButton) e.getSource();// checks which button was pressed
		if(source == saveButton) {
			canvas.save();
		} else if (source==loadButton) {
			canvas.load(); 
		} else if (source == sortButton) {//sort button got pressed
			canvas.sort();//calls sort() in drawPanel
		} else if(source == undoButton) {//undo was pressed
			canvas.undo();//calls undo() in drawPanel
		} else if(source == redoButton) {//redo was pressed
			canvas.redo();//calls redo() in drawPanel
		} else if (source == colorButton) {//the color picker button was pressed
			JColorChooser a = new JColorChooser();//sets up color chooser
			Color aColor=a.showDialog(null, "Pick a Color", Color.WHITE);//saves the color the user picks
			owner.setColor(aColor);//sends the color to the main program to set the current color
		} else if (source == squareButton) {//the square button was pressed
			owner.setBrush(new Square(0, 0, 0, 0));//creates a square for currentBrush in main
			owner.setBrushStroke(strokeSlider.getValue());//gets the stroke slider value
			owner.setBrushFill(fillCheck.isSelected());//checks if the fill box is checked
		} else if (source == circleButton) {//the circle button was pressed
			owner.setBrush(new Circle(0, 0, 0));//creates a circle for currentBrush in main
			owner.setBrushStroke(strokeSlider.getValue());//gets the stroke slider value
			owner.setBrushFill(fillCheck.isSelected());//checks if the fill box is checked
		} else {//the line button was pressed
			owner.setBrush(new Line(0, 0, 0, 0));//creates a line for currentBrush in main
			owner.setBrushStroke(strokeSlider.getValue());//gets the stroke slider value
			owner.setBrushFill(fillCheck.isSelected());//checks if the fill box is checked
		}
	}

	@Override
	// slider to change stroke weight of the brush
	public void stateChanged(ChangeEvent e) {
		int stroke = strokeSlider.getValue();//gets the slider value
		//System.out.println("Stroke weight "+ stroke);
		if (owner.getBrush() != null) {
			owner.setBrushStroke(stroke);//sets the currentBrush stroke to the slider value
		}
	}

	@Override
	// fill checkbox
	public void itemStateChanged(ItemEvent e) {
		//System.out.println("fill state change "+fillCheck.isSelected());
		if(owner.getBrush()!=null) {
			owner.setBrushFill(fillCheck.isSelected());//sets the fill value to the checkbox value
		}
		
	}
}