import java.awt.BasicStroke;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.geom.Ellipse2D;


/*Circle class, I need to rename this to ellipse
A shape subclass that holds the dimenstions for a ellipse

Danny Nguyen
*/
public class Circle extends Shape {
	private double width=0;
	private double height=0;

	//Circle Constructor
	//takes 2 ints for XY in super
	//takes a double for radius
	public Circle(int a, int b, double r) {
		super(a, b);
		if (r<0) throw new JavaPaintException("Negative radius");
		setSize((int) r);
	}
	//Ellipse Constructor
	//takes 2 ints for XY in super
	//takes 2 doubles for major and minor axis
	public Circle(int a, int b, double max, double w, double h) {
		super(a, b);
		if (height<0||width<0) throw new JavaPaintException("Negative axis");
		setW(w);
		setH(h);

	}

	//Copy Constructor
	public Circle(Circle o) {
		super(o.getX(), o.getY());
		setStroke(o.getStroke());
		setFill(o.getFill());
		setW(o.getW());
		setH(o.getH());
		setRGB(o.getRGB());
	}

	//Width getter and setter
	public double getW(){
		return width;
	}
	public void setW(double dw) {
		if(dw>=0) width=dw;
	}

	//Height getter and setter
	public double getH(){
		return height;
	}
	public void setH(double dh) {
		if(dh>=0) height=dh;
	}

	@Override
	//makes a circle
	public void setSize(int d) {
		setW((double)d);
		setH((double)d);
	}

	@Override
	//drawing ellipses
	public void draw(Graphics g) {
		Graphics2D g2d=(Graphics2D) g;
		g2d.setStroke(new BasicStroke(getStroke()));//gets stroke from super
		g2d.setColor(getRGB());//gets color from super
		if(getFill())g2d.fill(new Ellipse2D.Double(getX(), getY(), width, height));
		else g2d.draw(new Ellipse2D.Double(getX(), getY(), width, height));
	}
	@Override
	public double getArea() {
		return Math.PI*(width*height);
	}
	@Override
	public Shape clone() {
		return new Circle(this);
	}
	@Override
	//Comparing Shapes
	//smaller shapes are +1
	//larger shapes are -1
	public int compareTo(Object o) {
		int retval=0;//default comparison
		if(o==null || !(o instanceof Shape)) {//Comparable check
			throw new JavaPaintException("Object not comparable to circle");
		}

		//casts the object to be compared to a shape
		//since each shape calculates area differently
		Shape that;
		if(o instanceof Square) {
			that=(Square) o;
		}else if(o instanceof Circle){
			that=(Circle) o;
		}else that=(Line) o;

		if(this.getArea()==that.getArea()) {//2 shapes are equal
			retval=0;
		} else if(this.getArea()<that.getArea()) {//this is smaller
			retval=1;
		} else {
			retval=-1;//this is larger
		}
		return retval;
	}
}
