import java.awt.BorderLayout;
import java.awt.Color;

import javax.swing.JFrame;


/*
Main container for the entire JavaPaint program
This program holds the the various functionality needed for a paint program
Run using JavaPaintMain.java

Danny Nguyen
*/

public class JavaPaint extends JFrame{//JFrame to hold the JPanels
	private Shape currentBrush=new Shape(0,0);//saves the shape when the user presses a shape button
	private DrawPanel canvas=new DrawPanel(this);//the drawing area
	private ButtonPanel brushes=new ButtonPanel(this, canvas);//the area where all the brush buttons are located
	private Color currentColor=new Color(0,0,0);//saves the color between shapes being drawn and selected

	//Main constructor
	//sets up UI
	public JavaPaint() {
		super("JavaPaint by Danny Nguyen");
		setLayout(new BorderLayout());
		add(canvas, BorderLayout.CENTER);//drawing space in the center
		add(brushes, BorderLayout.WEST);//all utility buttons on the left
		setSize(1024, 768);
		setVisible(true);
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	//saves the shape the user clicks on
	//this function is called when the user presses one of the BrushPanel buttons
	public void setBrush(Shape pressed) {
		//cast so getArea() works
		if(pressed instanceof Square) currentBrush=(Square) pressed.clone();
		if(pressed instanceof Circle) currentBrush=(Circle) pressed.clone();
		if(pressed instanceof Line) currentBrush=(Line) pressed.clone();
		currentBrush.setRGB(currentColor);//sets brush to last selected color
		System.out.println("Current brush: " +currentBrush);//debug
	}

	//makes a copy of the current brush
	//then sends it to the drawPanel
	public Shape getBrush() {
		Shape retval=null;
		if(currentBrush!=null) retval=currentBrush;//null check
		//cast
		if(retval instanceof Square) retval = new Square((Square) retval);
		if(retval instanceof Circle) retval = new Circle((Circle) retval);
		if(retval instanceof Line) retval = new Line((Line) retval);
		retval.setRGB(currentColor);//set brush to last selected color
		return retval;
	}

	//sets the stroke of the brush
	//stroke should never be negative
	public void setBrushStroke(int s) {
		if(s>=0) currentBrush.setStroke(s);//invariant check
		System.out.println("shape stroke "+currentBrush.getStroke());//debug
	}
	//sets shape to draw outline or fill
	public void setBrushFill(boolean b) {
		currentBrush.setFill(b);
		System.out.println("shape fill " + currentBrush.getFill());//debug
	}
	//sets the shape's color
	public void setColor(Color o) {
		currentColor=new Color(o.getRed(),o.getGreen(),o.getBlue());//debug
	}
}
