import java.awt.BasicStroke;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.geom.Line2D;



/*Line class, this i don't need to rename
holds XY coordinates for start and end points of a line

Danny Nguyen
*/
public class Line extends Shape {
	private int x2;
	private int y2;

	//Main Constructor
	//takes the first 2 ints for start coordinates
	//takes the other 2 ints for end coordinates
	public Line(int a, int b, int c, int d) {
		super(a, b);
		setX2(c);
		setY2(d);
	}

	//Copy constructor
	public Line(Line o) {
		super(o.getX(), o.getY());
		setStroke(o.getStroke());
		setX2(o.getX2());
		setY2(o.getY2());
		setRGB(o.getRGB());
	}

	//X endpoint getter and setter
	public int getX2() {
		return x2;
	}
	public void setX2(int dx) {
		if(dx>=0) x2=dx;
	}

	//Y endpoint getter and setter
	public int getY2() {
		return y2;
	}
	public void setY2(int dy) {
		if(dy>=0) y2=dy;
	}


	@Override
	//area for this is calculated by the length of the line
	public double getArea() {
		//change in X squared
		double deltaX = getX()-getX2();
		deltaX = deltaX * deltaX;

		//change in Y squared
		double deltaY = getY()-getY2();
		deltaY = deltaY * deltaY;

		//sum of change in X and Y
		double retval = deltaX + deltaY;

		//square root of the sum
		retval = Math.sqrt(retval);
		return retval;
	}
	@Override
	//drawing lines, oh my
	public void draw(Graphics g) {
		Graphics2D g2d=(Graphics2D) g;
		g2d.setStroke(new BasicStroke(getStroke()));//gets stroke from super
		g2d.setColor(getRGB());//gets color from super
		g2d.draw(new Line2D.Float(getX(), getY(), getX2(), getY2()));
	}

	@Override
	public Shape clone() {//copy constructor
		return new Line(this);
	}
	@Override
	//Comparing Shapes
	//smaller shapes are +1
	//larger shapes are -1
	public int compareTo(Object o) {
		int retval=0;//default comparison
		if(o==null || !(o instanceof Shape)) {//comparable check
			throw new JavaPaintException("Object not comparable to line");
		}

		//casts the object to be compared to a shape
		//since each shape calculates area differently
		Shape that;
		if(o instanceof Square) {
			that=(Square) o;
		}else if(o instanceof Circle){
			that=(Circle) o;
		}else that=(Line) o;

		if(this.getArea()==that.getArea()) {//these are equal
			retval=0;
		} else if(this.getArea()<that.getArea()) {//this is smaller
			retval=1;
		} else {
			retval=-1;//this is larger
		}
		return retval;
	}

}
