import java.io.Serializable;

/*
 * Danny Nguyen
 * Creates and manages a List
 * implements Comparable to make 2D arrays possible with List, Stack, and QueueArray.java
 */
public class ListArray<tba extends Comparable> implements Comparable, Cloneable, Serializable{
	private Comparable[] List = new Comparable[10];//holds comparable data
	private int numElements = 0;//number of elements

	//default constructor
	public ListArray() {
	}

	//copy constructor
	public ListArray(ListArray o) {
		reSize(o.size());
		for(int i=0; i<o.size(); i++) {
			add((tba) o.get(i));
		}
	}

	// adds an object at numElements
	// then iterates numElements
	public void add(tba o) {
		if (o == null) // function does nothing if handed null
			return;
		if (numElements == List.length)// increases Stack[] size if needed
			reSize(numElements + 10);
		List[numElements++] = o;//adds to the list
	}

	// adds an object at specified index
	// then iterates numElements
	public void add(tba o, int index) {
		if (o == null) // function does nothing if handed null
			return;
		if(index>List.length-1) throw new RuntimeException("index outside of List length");
		if (numElements == List.length)// increases List[] size if needed
			reSize(numElements + 10);
		ShiftRight(index);//shifts elements to make room
		List[index] = o;//adds to the list
		numElements++;//increment element count
	}

	// returns an object from List[] at numElements-1
	public tba remove(int index) {
		// null check
		if (List[index] == null)//null check
			throw new RuntimeException("element at " + index + "is null");
		if (numElements < List.length - 10)// decreases List[] size if needed
			reSize(numElements);
		Object retVal = List[index];//holds the object at the index
		ShiftLeft(index);//shifts the elements
		numElements--;//decrememnts element count
		return (tba) retVal;//returns the object
	}

	//searches the list for a specified object
	//and removes that object from the list
	public tba remove(tba o) {
		return remove(indexOf(o));//finds the object and calls the other remove
	}

	//grabs an object in the list without removing it
	public tba get(int index) {
		if (List[index] == null||index<0||index>size()-1)
			throw new RuntimeException("element at " + index + "is null");
		return (tba) List[index];
	}

	//sorting the list
	public void sort() {
		new QuickSort(List, size()-1);
	}

	// checks if the List is empty
	public boolean isEmpty() {
		return numElements == 0;
	}

	// returns number of elements in the List
	public int size() {
		return numElements;
	}

	//swaps 2 elements
	private void swap(int index1, int index2) {
		Comparable temp=List[index1];
		List[index1]=List[index2];
		List[index2]=temp;
	}

	//finds the index of a specific object
	public int indexOf(Object o) {
		int retVal = -1;//object not found
		if (o == null)// null check
			return -1;
		for (int i = 0; i < numElements; i++) {// walks through Lists
			// checks for matching object
			if (List[i].equals(o)) {
				retVal = i;
				break;
			}
		}
		return retVal;
	}

	//walks the list
	//concatanating all the elements
	public String toString() {
		String retVal = "";
		for (int i = 0; i < numElements; i++) {
			retVal += List[i] + " ";
		}
		return retVal;
	}

	// resizes List to fit more objects or shrinks to save memory
	private void reSize(int newSize) {
		Comparable[] temp = new Comparable[newSize];// creates a new temp array
		for (int i = 0; i < numElements; i++) {
			temp[i] = List[i]; // copies data from List to temp
		}
		List = temp; // assign temp to List
	}

	// Shifts List[] left when remove is called
	private void ShiftLeft(int index) {
		for (int i = index; i < numElements - 1; i++) {
			List[i] = List[i + 1];
		}
	}

	// Shifts List[] right when add is called
	private void ShiftRight(int index) {
		for (int i = numElements; i > index; i--) {
			List[i] = List[i - 1];
		}
	}

	@Override
	public int compareTo(Object o) {
		// TODO Auto-generated method stub
		return 0;
	}
	@Override
	public Object clone() {
		return new ListArray(this);
	}
}