/*
The quicksort
Sorts an array of comparables
the program loops through the array, checking each element against a pivot
all values less than the pivot are moved to the left, all values greater than to the righ
then recursivly sorts each half of the array until it gets to a subarray of 1

Danny Nguyen
*/
public class QuickSort extends Sort {
	public QuickSort(Comparable[] data, int max) {
		sort(data, 0, max);
	}

	//Selects a pivot value for quicksorting
	//gets the value at the index in the middle of the unsorted partition
	private Comparable selectPivot(Comparable[] data, int low, int high) {
		return data[(low + high) / 2];
	}

	//Sorts the set of data
	//rearranges the partion so all values < pivot is to the left and to the right for > pivot
	private int partition(Comparable data[], int left, int right) {
		int i = left, j = right; //holds the limit to be manipulated
		Comparable pivot = selectPivot(data, left, right);//value to be compared
		while (i <= j) {//keeps the left and right cursor from crossing
			//finds a value greater than the pivot to the left
			while (data[i].compareTo(pivot) < 0) {
				incrementComparisons();
				i++;//moves the left cursor right
			}
			incrementComparisons();
			//finds a value less than the pivot to the right
			while (data[j].compareTo(pivot) > 0){
				incrementComparisons();
				j--;//moves the right cursor left
			}
			incrementComparisons();
			//swaps the values
			if (i <= j) {
				swap(data, i, j);
				i++;
				j--;
			}
		}
		return i;//returns the middle index
	}

	@Override
	public void sort(Comparable data[], int left, int right) {
		if(data==null||data.length==1) return;//data set of 1 or null is already sorted
		int index = partition(data, left, right);//sorts the data set
		if (left < index - 1)
			sort(data, left, index - 1);//sorts left subset
		if (index < right)
			sort(data, index, right);//sorts right subset
	}

}