import java.awt.*;
import java.io.Serializable;

/* Class Shape
This is the superclass for all shapes(Lines, Squares, and Circles)
Contains the starting XY coordinates, stroke weight, fill, and RGB color
all shapes comparable via int area

Danny Nguyen
 */

public class Shape implements Cloneable, Serializable, Comparable {
	private int x;
	private int y;
	private int stroke=1;
	private boolean fill=false;
	private Color rgb=new Color(0,0,0);

	//Main constructor
	//takes 2 ints for XY coordinates
	public Shape( int a, int b ) {
		if(a<0||b<0) throw new JavaPaintException("Negative coordinate value");
		setX(a);
		setY(b);
	}

	//copy constructor
	public Shape(Shape o) {
		setX(o.getX());
		setY(o.getY());
		setStroke(o.getStroke());
		setFill(o.getFill());
		setRGB(o.getRGB());
	}

	//X getter and setter
	public int getX() {
		return x;
	}
	public void setX(int dx) {
		if(dx>=0) x=dx;
	}

	//Y getter and setter
	public int getY() {
		return y;
	}
	public void setY(int dy) {
		if(dy>=0) y=dy;
	}

	//Stroke getter and setter
	public void setStroke(int s) {
		if(s>=0) stroke=s;
	}
	public int getStroke() {
		return stroke;
	}

	//fill getter and setter
	public void setFill(boolean b) {
		fill=b;
	}
	public boolean getFill() {
		return fill;
	}

	//Color getter and setter
	public Color getRGB() {
		return new Color(rgb.getRGB());
	}
	public void setRGB(Color n) {
		rgb=new Color(n.getRed(),n.getGreen(),n.getBlue());
	}

	//to be overridden
	public double getArea() {
		return 0;
	}

	//to be overridden
	public void draw(Graphics g) {
	}

	//to be overridden
	public void setSize(int size) {
	}

	@Override
	//Shape objects are equal if they share the same XY coordinates
	public boolean equals(Object o) {
		if(o==null||!(o instanceof Shape)) return false;
		Shape that=(Shape) o;
		boolean retval=false;
		if(this.getX()==that.getX()) {
			if(this.getY()==that.getY()) retval=true;
		}
		return retval;
	}

	@Override
	public int compareTo(Object o) {
		return 0;
	}
	@Override
	public Shape clone() {
		return new Shape(this);
	}
}